#! /bin/env python3
#

from handlers import init_handlers
from processThread import processThread
from socketThread import socketThread
import ns_utils
import time
import logging
import config
import ncurses
import pyqt

log = logging.getLogger('')

pthread = processThread(1, "processThread")
sthread = socketThread(2, "socketThread", pthread.recv)
init_handlers(pthread)
pthread.socketThread = sthread
sthread.processThread = pthread
sthread.start()
while not sthread.running:
  time.sleep(0.1)
pthread.start()
while not pthread.authed:
  time.sleep(0.1)
ns_utils.change_status(pthread, "actif")
time.sleep(1)

try:
  if config.args.do_ncurses:
    ncurses.main(pthread)
  elif config.args.do_pyqt:
    pyqt.main(pthread)
  else:
    while (True):
      time.sleep(1)
      cmd = input("Cmd>")
      pthread.add_queueout(cmd)
except (KeyboardInterrupt, SystemExit):
  log.info("Exiting")
except:
  log.critical("Exception caught")
  raise
finally:
  pthread.add_queueout("exit")
  while not pthread.q_out.empty():
    time.sleep(1)
  pthread.stop()
  sthread.stop()
  pthread.join()
  sthread.join()



import configparser
import os
import logging
import argparse

aparser = argparse.ArgumentParser(description='Python full-scale client for'\
    +'   NetSoul by lastmikoi')
aparser.add_argument('--debug', dest='lvl', action='store_const',\
    const=logging.DEBUG, default=logging.INFO, help='turns on DEBUG level logging')
aparser.add_argument('--logtostderr', dest='to_file', action='store_const',\
    const=False, default=True, help='display logging messages to stderr instead'\
    +'of writing to file')
aparser.add_argument('--ncurses', dest='do_ncurses', action='store_const',\
    const=True, default=False, help='run with ncurses interface')
aparser.add_argument('--pyqt', dest='do_pyqt', action='store_const',\
    const=True, default=False, help='run with PyQt interface')
args = aparser.parse_args()

FORMAT = \
    "[%(levelname)s]%(asctime)s|%(threadName)s:%(message)s|%(filename)s@Line#%(lineno)d"

if args.to_file:
  logging.basicConfig(filename=os.environ['HOME'] + "/.lastsoul.log",level=args.lvl,format=FORMAT)
else:
  logging.basicConfig(level=args.lvl,format=FORMAT)
std_answer = r"rep (\d+) -- ([^\Z]+)"
usercmd_handler = r"user_cmd "\
    +r"(\d+):(\w+):(\d+)/(\d+):(\w+)@([^:]+):([^:]+):([^:]+):(\w+) "

def get_config(section_name, item_name):
  try:
    config = configparser.ConfigParser()
    config.read(os.environ['HOME'] + '/.lastsoul.cfg')
    ret = config.get(section_name, item_name)
  except configparser.Error:
    return (None)
  return (ret)



#! /bin/env python
#

import configparser
import os

def defl_input(prompt, default):
  usr = input(prompt+"(%s)" % default);
  if len(usr):
    return (usr)
  else:
    return (default)

def main():
  print("Welcome to lastSoul's netSoul client configurator.")
  print("	!!! Beware, this tool will delete your old configuration !!!")
  input("	Press enter to proceed.")
  config = configparser.RawConfigParser()
  config.add_section("Connection")
  config.set("Connection", 'server', defl_input("NetSoul server IP/Hostname ? ", 'ns-server.epitech.net'))
  config.set("Connection", 'port', defl_input("NetSoul server port ? ", '4242'))
  config.add_section("Auth")
  config.set("Auth", 'login', defl_input("Login ? ", None))
  config.set("Auth", 'passwd', defl_input("Password socks ? ", None))
  config.add_section("Misc")
  config.set("Misc", 'userdata', defl_input("UserData ? ", "lastSoul v1.2"))
  config.set("Misc", 'location', defl_input("Location ? ", None))
  config.set("Misc", 'watch', defl_input("Watchlist (comma-separated) ? ", None))
  try:
    with open(os.environ['HOME'] + '/.lastsoul.cfg', 'wb') as configfile:
      config.write(configfile)
      print("Configuration saved.")
  except IOError as e:
    print("Error while saving config : %s" % e)

if __name__ == "__main__":
  main()



import urllib.request, urllib.parse, urllib.error
import time
import socket
import queue
import re
import config
from threading import Condition
from config import std_answer, usercmd_handler
import logging

log = logging.getLogger('')

def change_status(thread, newstatus):
  newstatus = urllib.parse.quote(newstatus)
  thread.add_queueout("state %s" % newstatus)

def send_msg(thread, username, msg):
  username = username.split(' ')[0]
  msg = urllib.parse.quote(msg)
  thread.add_queueout("user_cmd msg_user %s msg %s" % (username, msg))

def watch_user(thread, usernames, callback):
  usercmd_userscoped_handler = r"user_cmd "\
      +r"(\d+):(\w+):(\d+)/(\d+):(%s)@([^:]+):([^:]+):([^:]+):(\w+) " % '|'.join(usernames)
  watchhandler = usercmd_userscoped_handler + r"\| (login|logout|state)([^\Z]+)\Z"
  thread.register_handler(watchhandler, callback)
  thread.add_queueout("user_cmd watch_log_user {%s}" % ','.join(usernames))

def get_status(thread, usernames):
  finished = Condition()
  userinfo = []

  def finished_recv(thread, m):
    thread.forget_handler(finished_recv)
    thread.forget_handler(internal_recv)
    finished.acquire()
    finished.notify()
    finished.release()
    pass

  def internal_recv(thread, m):
    userinfo.append(dict(socknum=m.group(1), username=m.group(2), ip=m.group(3),
      login_time=int(m.group(4)), update_time=int(m.group(5)),
      low_t=int(m.group(6)), hi_t=int(m.group(7)), wtype=m.group(8),
      location=m.group(9), group=m.group(10), status=m.group(11),
      userdata=m.group(12)))
    pass

  handler = r"(\d+) (\w+) ([^ ]+) (\d+) (\d+) (\d+) (\d+) ([^ ]+) ([^ ]+) (\w+) ([^ ]+) ([^ ]+)"
  thread.register_handler(handler, internal_recv)
  thread.register_handler(std_answer, finished_recv)
  thread.add_queueout("list_users {%s}" % ','.join(usernames))
  finished.acquire()
  finished.wait()
  finished.release()
  return (userinfo)



#!/usr/bin/python
# -*- coding: latin9 -*-

from PyQt4 import QtGui, QtCore
from queue import Queue
from config import usercmd_handler, get_config
from _version import __version__
from time import gmtime, strftime
import ns_utils
import urllib.request, urllib.parse, urllib.error
import logging

log = logging.getLogger('')

class UIbridge(QtCore.QThread):
  def __init__(self):
    super(UIbridge, self).__init__()
    self.q_exec = Queue()
    self.mutex = QtCore.QMutex()
    self.cond = QtCore.QWaitCondition()
    self.running = True
    self.wait = False
    pass

  def __del__(self):
    self.wait()
    pass

  def add_to_queue(self, signal, *args):
    self.mutex.lock()
    self.q_exec.put((signal, args))
    self.wait = False
    self.cond.wakeAll()
    self.mutex.unlock()
    pass

  def process(self, data):
    signal = data[0]
    args = data[1]
    signal.emit(*args)
    pass

  def run(self):
    while self.running:
      self.mutex.lock()
      if not self.q_exec.empty():
        data = self.q_exec.get()
        self.mutex.unlock()
        self.process(data)
      else:
        self.mutex.unlock()
      if self.wait:
        self.mutex.lock()
        self.cond.wait(self.mutex)
        self.mutex.unlock()
      self.wait = True
    pass

class mainWindow(QtGui.QMainWindow):
  addTab_signal = QtCore.pyqtSignal(dict)

  def __init__(self, title, thread):
    super(mainWindow, self).__init__()
    self.thread = thread
    self.initUI(title)
    self.tablist = []
    self.bridge = UIbridge()
    self.bridge.start()
    pass

  def send(self):
    title = self.tabs.tabText(self.tabs.currentIndex())
    if title is not "Home":
      widgets = None
      for username, curwidgets in self.tablist:
        if username == title:
          widgets = curwidgets
          break
        pass
      msgbar = widgets['msgbar']
      msg = msgbar.text()
      ns_utils.send_msg(self.thread, title, msg)
      msgbar.setText('')
      recvzone = widgets['recvzone']
      curtime = strftime("[%H:%M:%S]", gmtime())
      echo = "->(" + curtime + " " + title + ") " + msg
      recvzone.append(echo)
      self.show()
    pass

  def newMsgTab(self, title):
    main = QtGui.QGroupBox()
    layout = QtGui.QGridLayout()
    layout.setSpacing(10)

    recvzone = QtGui.QTextEdit(main)
    recvzone.setReadOnly(True)
    layout.addWidget(recvzone, 0, 0, 9, 2)

    msgbar = QtGui.QLineEdit(main)
    sendbutton = QtGui.QPushButton("Send", main)
    msgbar.connect(msgbar, QtCore.SIGNAL("returnPressed()"), self.send)
    sendbutton.clicked.connect(self.send)

    layout.addWidget(msgbar, 10, 0)
    layout.addWidget(sendbutton, 10, 1)
    main.setLayout(layout)

    self.tabs.addTab(main, title)
    widgets = dict(main=main, recvzone=recvzone, sendbutton=sendbutton,
        msgbar=msgbar)
    return (widgets)

  def addTab(self, msgdata):
    widgets = None
    for username, curwidgets in self.tablist:
      if username == msgdata['username']:
        widgets = curwidgets
        break
      pass
    curtime = strftime("[%H:%M:%S]", gmtime())
    msg = curtime + " " + msgdata['username'] + ":" + urllib.parse.unquote(msgdata['msg'])
    if not widgets:
      widgets = self.newMsgTab(msgdata['username'])
      self.tablist.append((msgdata['username'], widgets))
      pass
    widgets['recvzone'].append(msg)
    self.show()
    pass

  def recv_private_msg(self, thread, m):
    msgdata = dict(socknum=int(m.group(1)), usertype=m.group(2), low_t=int(m.group(3)),\
        hi_t=int(m.group(4)), username=m.group(5), ip=m.group(6),\
        workstation=m.group(7), location=m.group(8),\
        group=m.group(9), msg=m.group(10))

    try:
      self.addTab_signal.disconnect(self.addTab)
    except TypeError:
      pass
    self.addTab_signal.connect(self.addTab)
    self.bridge.add_to_queue(self.addTab_signal, msgdata)
    pass

  def updateUserSpace(self, userinfowidget, username):
    userinfowidget.setTitle(username)
    children = userinfowidget.findChildren(QtGui.QLabel)
    try:
      userinfo = ns_utils.get_status(self.thread, (str(username),))[0]
    except IndexError:
      userinfo = None
    for i, child in enumerate(children):
      if child.__class__.__name__ == "QLabel":
        title = child.text().split(' :')[0]
        if userinfo:
          info = userinfo[str(title)]
        else:
          info = "Disconnected"
        child.setText(title + " :  " + str(info))
      pass

  def handle_listclick(self, item):
    username = item.text()
    self.updateUserSpace(self.userinfo, username)
    pass

  def handle_listdoubleclick(self, item):
    listusername = item.text()
    widgets = None
    for username, curwidgets in self.tablist:
      if username == listusername:
        widgets = curwidgets
        break
      pass
    if not widgets:
      widgets = self.newMsgTab(listusername)
      self.tablist.append((listusername, widgets))
    self.tabs.setCurrentWidget(widgets['main'])

  def initUI(self, title):
    def initHome(self):
      def initUserSpace(self, parent):
        layout = QtGui.QGridLayout()
        layout.setSpacing(25)
        layout.addWidget(QtGui.QLabel("socknum :", parent), 0, 0)
        layout.addWidget(QtGui.QLabel("username :", parent), 1, 0)
        layout.addWidget(QtGui.QLabel("ip :", parent), 0, 1)
        layout.addWidget(QtGui.QLabel("login_time :", parent), 2, 0)
        layout.addWidget(QtGui.QLabel("update_time :", parent), 2, 1)
        layout.addWidget(QtGui.QLabel("low_t :", parent), 5, 0)
        layout.addWidget(QtGui.QLabel("hi_t :", parent), 5, 1)
        layout.addWidget(QtGui.QLabel("wtype :", parent), 6, 0)
        layout.addWidget(QtGui.QLabel("location :", parent), 3, 0)
        layout.addWidget(QtGui.QLabel("group :", parent), 1, 1)
        layout.addWidget(QtGui.QLabel("status :", parent), 4, 0, 1, 3)
        layout.addWidget(QtGui.QLabel("userdata :", parent), 3, 1)
        parent.setLayout(layout)
        pass

      self.home = QtGui.QGroupBox(title)
      self.homelayout = QtGui.QGridLayout()
      self.homelayout.setSpacing(5)

      self.selfinfo = QtGui.QGroupBox("You", self.home)
      initUserSpace(self, self.selfinfo)

      self.userinfo = QtGui.QGroupBox("username", self.home)
      initUserSpace(self, self.userinfo)

      self.homelayout.addWidget(self.selfinfo, 0, 0, 1, 10)
      self.homelayout.addWidget(self.userinfo, 1, 0, 1, 10)

      self.friendlist = QtGui.QListWidget(self.home)
      friends = get_config("Misc", "watch").split(", ")
      self.friendlist.insertItems(0, friends)
      self.friendlist.itemClicked.connect(self.handle_listclick)
      self.friendlist.itemDoubleClicked.connect(self.handle_listdoubleclick)
      self.homelayout.addWidget(self.friendlist, 0, 11, 2, 1)
      self.home.setLayout(self.homelayout)
      pass

    initHome(self)
    self.updateUserSpace(self.selfinfo, get_config("Auth", "login"))
    self.tabs = QtGui.QTabWidget()
    self.tabs.addTab(self.home, "Home")

    close = QtGui.QAction(QtGui.QIcon('ressources/application-exit-2.png'), 'Close', self)
    close.triggered.connect(QtGui.qApp.quit)

    self.bar = self.addToolBar('Actions')
    self.bar.addAction(close)

    self.setGeometry(100, 100, 800, 600)
    self.setWindowTitle(title)
    self.setCentralWidget(self.tabs)
    privmsg_handler = usercmd_handler + r"\| msg (\S+) dst=(\S+)"
    self.thread.register_handler(privmsg_handler, self.recv_private_msg)
    self.show()
    pass
  pass

def main(thread):
  app = QtGui.QApplication([])
  win = mainWindow("lastSoul v%s" % __version__, thread)
  app.exec_()
  pass

if __name__ == '__main__':
  main(None)



import threading
import queue
from config import get_config
import socket
import time
import logging

log = logging.getLogger('')

class outThread(threading.Thread):
  def __init__(self, socket, socketLock):
    super(outThread, self).__init__()
    log.debug("Creating")
    self.name = "socketThread:outThread"
    self.running = True
    self.s = socket
    self.s_lock = socketLock
    self.q_out = queue.Queue()
    self.q_outLock = threading.Lock()
    self.waitLock = threading.Condition()
    pass

  def wakeup(self):
    self.waitLock.acquire()
    self.waitLock.notify()
    self.waitLock.release()

  def run(self):
    log.debug("Starting")
    while self.running:
      self.q_outLock.acquire()
      if not self.q_out.empty():
        data = bytes(self.q_out.get(), 'ascii')
        log.info("Sending [%s\\n]" % data[:-1])
        self.q_outLock.release()
        self.s_lock.acquire()
        self.s.sendall(data)
        self.s_lock.release()
      else:
        self.q_outLock.release()

      self.waitLock.acquire()
      self.waitLock.wait()
      self.waitLock.release()
    log.debug("Exiting")
    pass

class inThread(threading.Thread):
  def __init__(self, socket, socketLock, recv_handler):
    super(inThread, self).__init__()
    log.debug("Creating")
    self.name = "socketThread:inThread"
    self.running = True
    self.s = socket
    self.s_lock = socketLock
    self.recv_handler = recv_handler
    pass

  def run(self):
    log.debug("Starting")
    while self.running:
      self.s_lock.acquire()
      self.s.setblocking(False)
      data = None
      try:
        data = str(self.s.recv(1024), 'ascii')
        self.s.setblocking(True)
        self.s_lock.release()
      except socket.error:
        self.s.setblocking(True)
        self.s_lock.release()
        time.sleep(0.1)
      if data:
        for line in data.split('\n'):
          if line:
            log.info("Recieved [%s\\n]" % line)
            self.recv_handler(line)
    log.debug("Exiting")
    pass

class socketThread(threading.Thread):
  def __init__(self, threadID, name, recv_handler):
    super(socketThread, self).__init__()
    log.debug("Creating")
    self.daemon = True
    self.running = False
    self.processThread = None
    self.threadID = threadID
    self.name = name
    self.s = self.init_socket()
    self.s_lock = threading.Lock()
    self.recv_handler = recv_handler
    self.q_out = queue.Queue()
    pass

  def init_socket(self):
    s = None
    HOST = get_config("Connection", "server")
    PORT = int(get_config("Connection", "port"))
    log.info("Connecting to %s:%s" % (HOST, PORT))
    for res in socket.getaddrinfo(HOST, PORT, socket.AF_UNSPEC, socket.SOCK_STREAM):
      af, socktype, proto, canonname, sa = res
      try:
        s = socket.socket(af, socktype, proto)
      except socket.error as msg:
        s = None
        continue
      try:
        s.connect(sa)
      except socket.error as msg:
        s.close()
        s = None
        continue
      break
    if s is None:
      raise IOError("Couln't connect to server")
    return (s)

  def add_queueout(self, data):
    self.outThread.q_outLock.acquire()
    self.outThread.q_out.put(data)
    self.outThread.q_outLock.release()
    self.outThread.wakeup()
    pass

  def stop(self):
    self.inThread.running = False
    self.outThread.running = False
    self.outThread.wakeup()

  def run(self):
    log.debug("Starting")
    self.outThread = outThread(self.s, self.s_lock)
    self.inThread = inThread(self.s, self.s_lock, self.recv_handler)
    self.running = True
    while not self.processThread.running:
      time.sleep(0.1)
    self.outThread.start()
    self.inThread.start()
    self.inThread.join()
    self.outThread.join()
    self.s.close()
    log.debug("Exiting")



import re
import hashlib
import urllib.request, urllib.parse, urllib.error
from config import get_config, std_answer, usercmd_handler
import logging

log = logging.getLogger('')

def recv_private_msg(thread, m):
  msgdata = dict(socknum=int(m.group(1)), usertype=m.group(2), low_t=int(m.group(3)),\
      hi_t=int(m.group(4)), username=m.group(5), ip=m.group(6),\
      workstation=m.group(7), location=m.group(8),\
      group=m.group(9), msg=m.group(10))
  log.info("Message recieved from "+ msgdata['username'] + "@" +\
    msgdata['location'] + ":" + msgdata['msg'])

def recv_dotnetSoul(thread, m):
  msgdata = dict(socknum=int(m.group(1)), usertype=m.group(2), low_t=int(m.group(3)),\
      hi_t=int(m.group(4)), username=m.group(5), ip=m.group(6),\
      workstation=m.group(7), location=m.group(8),\
      group=m.group(9), action=m.group(10), unused1=m.group(11))
  log.info("User " + msgdata['username'] + " " + msgdata['action'])

def recv_ping(thread, m):
  answer = int(m.group(1))
  log.info("Sending back ping %d" % answer)
  thread.add_queueout("ping %d " % answer)

def recv_auth(thread, m):
  auth = None

  def calc_auth(auth):
    username = get_config("Auth", "login")
    passwd = get_config("Auth", "passwd")
    tohash = "%s-%s/%s%s" % (auth['md5'], auth['myip'], auth['myport'], passwd)
    repmd5 = hashlib.md5(tohash.encode('ascii')).hexdigest()
    udata = urllib.parse.quote(get_config("Misc", "userdata"))
    userlocation = urllib.parse.quote(get_config("Misc", "location"))
    return (dict(username=username, md5=repmd5, userdata=udata, location=userlocation))

  def stage_two(thread, m):
    if m.group(1) != '002':
      raise ValueError(m.group(2))
    thread.forget_handler(stage_two)
    answ = calc_auth(auth)
    msg = "ext_user_log %s %s %s %s"\
      % (answ['username'], answ['md5'], answ['location'], answ['userdata'])
    thread.register_handler(std_answer, stage_three)
    thread.add_queueout(msg)
    pass

  def stage_three(thread, m):
    if m.group(1) != '002':
      raise ValueError(m.group(2))
    thread.forget_handler(stage_three)
    log.info("Authed sucessfully")
    thread.add_queueout("state connection")
    thread.authed = True
    pass

  auth = dict(socknum=int(m.group(1)), md5=m.group(2), myip=m.group(3),
    myport=int(m.group(4)), timestmp=int(m.group(5)))
  thread.register_handler(std_answer, stage_two)
  thread.add_queueout("auth_ag ext_user none none")
  pass


def init_handlers(thread):
  auth_handler = r"salut (\d+) (\w+) (\S+) (\d+) (\d+)"
  ping_handler = r"ping (\d+)"
  privmsg_handler = usercmd_handler + r"\| msg (\S+) dst=(\S+)"
  dotnetSoul_handler = usercmd_handler + r"\| dotnetSoul_(\S+) (\S+) dst=(\S+)"

  thread.register_handler(auth_handler, recv_auth)
  thread.register_handler(privmsg_handler, recv_private_msg)
  thread.register_handler(ping_handler, recv_ping)
  thread.register_handler(dotnetSoul_handler, recv_dotnetSoul)



#! /bin/env python
#

from config import usercmd_handler
from _version import __version__
import config
import curses
import urllib.request, urllib.parse, urllib.error
import time
import sys
import select
import logging
import ns_utils

log = logging.getLogger('')

def main(thread):
  msgs = []
  dest = []
  usermsg = []
  friends = []
  field = 0

  def refresh():
    stdscr.refresh()
    statuswin.refresh()
    txtwin.refresh()
    dstwin.clear()
    dstwin.addstr(0, 0, ''.join(dest))
    dstwin.refresh()
    msgwin.erase()
    msgwin.addstr(0, 0, ''.join(usermsg))
    msgwin.refresh()
    friendswin.clear()
    friendswin.border(curses.ACS_VLINE, curses.ACS_VLINE,
        curses.ACS_HLINE, curses.ACS_HLINE,
        curses.ACS_TTEE, curses.ACS_RTEE,
        curses.ACS_BTEE, curses.ACS_RTEE)
    for i, friend in enumerate(friends):
      friendswin.addstr(1 + i, 1, friend['username'] + ':' + friend['status'])
    friendswin.refresh()

  def sendmsg(thread):
    ns_utils.send_msg(thread, ''.join(dest), ''.join(usermsg))
    refresh()

  def recv_watch(thread, m):
    groups = m.groups()
    newstatus = None
    if groups[9] == 'state':
      newstatus = groups[10].split(':')[0]
    elif groups[9] == 'logout':
      newstatus = "logout"
    else:
      newstatus = "connecting"
    friendinfo = dict(socknum=int(groups[0]), usertype=groups[1],\
        low_t=int(groups[2]), hi_t=int(groups[3]), username=groups[4],\
        ip=groups[5], wtype=groups[6], location=groups[7], usergroup=groups[8],\
        status=newstatus)
    for i, friend in enumerate(friends):
      if friend['username'] == friendinfo['username']:
        log.debug("Updating status of %s" % friendinfo['username'])
        if newstatus == "logout":
          friends.pop(i)
        else:
          friends[i] = friendinfo
        refresh()
        return
    log.debug("New status for %s" % friendinfo['username'])
    if newstatus != "logout":
      friends.append(friendinfo)
    refresh()

  def recv_private_msg(thread, m):
    msgdata = dict(socknum=int(m.group(1)), usertype=m.group(2), low_t=int(m.group(3)),\
        hi_t=int(m.group(4)), username=m.group(5), ip=m.group(6),\
        workstation=m.group(7), location=m.group(8),\
        group=m.group(9), msg=m.group(10))
    msgs.append(msgdata)
    while len(msgs) >= txtwin.getmaxyx()[0] - 1:
      msgs.pop(0)
    for i, msg in enumerate(msgs):
      txtwin.addstr(1 + i, 1, "(%s):%s" % (msg['username'], urllib.parse.unquote(msg['msg'])))
    refresh()

  privmsg_handler = usercmd_handler + r"\| msg (\S+) dst=(\S+)"
  thread.register_handler(privmsg_handler, recv_private_msg)
  try:
    stdscr = curses.initscr()
    curses.noecho(); stdscr.keypad(1); curses.cbreak()
    curses.curs_set(0)
    stdscr.border()
    size = stdscr.getmaxyx()
    statuswin = curses.newwin(1, size[1] - 2, 1, 1)
    statuswin.addstr(0, 0, "lastSoul v%s" % __version__, curses.A_BOLD)
    txtwin = curses.newwin(size[0] - 4, 0, 2, 0)
    txtwin.border(curses.ACS_VLINE, curses.ACS_VLINE,\
        curses.ACS_HLINE, curses.ACS_HLINE,\
        curses.ACS_LTEE, curses.ACS_RTEE,\
        curses.ACS_LTEE, curses.ACS_RTEE)
    msgwin = curses.newwin(1, int(size[1] / 4 * 3 - 1), size[0] - 2, int(size[1] / 4))
    dstwin = curses.newwin(1, int(size[1] / 4 - 2), size[0] - 2, 1)
    friendswin = curses.newwin(txtwin.getmaxyx()[0], 0,\
        2, int(size[1] / 4 * 3 + 2))
    oldtimestamp = 0
    while (True):
      timestamp = time.time()
      refresh()
      try:
        fd = select.select([sys.stdin], [], [], 0.5)[0]
      except select.error:
        fd = None
      if fd:
        c = fd[0].read(1)
        if ord(c) == 0x7F:
          if field and len(dest):
            dest.pop()
          else:
            if len(usermsg):
              usermsg.pop()
        elif ord(c) == 0x9:
          field = not field
        elif ord(c) == 0xD:
          sendmsg(thread)
          usermsg = []
        else:
          if field:
            dest.append(c)
          else:
            usermsg.append(c)
      if (oldtimestamp < timestamp - 60):
        statusresp = ns_utils.get_status(thread,
            (config.get_config("Auth", "login"),))
        for userinfo in statusresp:
          statussize = statuswin.getmaxyx()
          statuswin.addstr(0, statussize[1] - 1 - len(userinfo['status']),
              userinfo['status'])
        oldtimestamp = timestamp
  except Exception as e:
    log.critical("Exception caught : %s" % e)
    raise e
  finally:
    curses.nocbreak(); stdscr.keypad(0); curses.echo()
    curses.endwin()




# -*- coding: iso-8859-15 -*-
from distutils.core import setup
from lastsoul._version import __version__

setup(
    name = "lastSoul",
    requires = ["PyQT",],
    version = __version__,
    packages = ["lastsoul"],
    license = "GNU General Public License v3 (GPLv3)",
    description = "Python (not yet)full scale client for IONIS's students (EPITECH/EPITA/etc)",
    author = "Mickaël FALCK",
    author_email = "lastmikoi+py@gmail.com",
    url = "https://bitbucket.org/lastmikoi/lastsoul",
    keywords = ["netsoul", "ncurses", "pyqt", "epitech", "ionis"],
    classifiers = [
      "Programming Language :: Python :: 3",
      "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
      "Operating System :: OS Independent",
      "Development Status :: 4 - Beta",
      "Environment :: X11 Applications :: Qt",
      "Environment :: Console :: Curses",
      "Environment :: Console",
      "Intended Audience :: End Users/Desktop",
      "Topic :: Communications :: Chat",
      ],
    long_description = """\
Python (not yet)full scale client for IONIS's students (EPITECH/EPITA/etc)
----------------------------------------

Features :
- Connection to netsoul server and authentification using config file
- Configuration generator script
- Multithreaded queuing system for processing in and out operations without blocking calls.
- Advanced handling with registration system of function called when regexp match with input
- Severals UI:
  * Raw command mode : allow to type commands directly to the server while being authed
  * PyQt Interface : with multi-tab, and userlist support
  * ncurses Interface : with user tracking
"""
)



from socketThread import socketThread
import threading
import re
import queue
import logging

log = logging.getLogger('')

class	processThread(threading.Thread):
  def __init__(self, threadID, name):
    super(processThread, self).__init__()
    log.debug("Creating")
    self.daemon = True
    self.running = False
    self.socketThread = None
    self.threadID = threadID
    self.name = name
    self.wait = False
    self.authed = False
    self.q_in = queue.Queue()
    self.q_out = queue.Queue()
    self.q_inLock = threading.Lock()
    self.q_outLock = threading.Lock()
    self.funcsLock = threading.Lock()
    self.waitLock = threading.Condition()
    self.inputs = []
    self.funcs = []
    pass

  def process_in(self, data):
    self.inputs.append(data)
    pass

  def process_out(self, data):
    self.socketThread.add_queueout(data+'\n')
    pass

  def register_handler(self, match, func):
    self.funcsLock.acquire()
    log.debug("func %s registered" % func)
    self.funcs.append([match, func])
    self.funcsLock.release()
    pass

  def forget_handler(self, func):
    self.funcsLock.acquire()
    for i, (match, funcname) in enumerate(self.funcs):
      if func is funcname:
        self.funcs.pop(i)
        log.debug("%s forgotten" % func)
        break
    self.funcsLock.release()

  def add_queuein(self, data):
    if len(data):
      self.q_inLock.acquire()
      self.q_in.put(data)
      self.q_inLock.release()
      self.wakeup()

  def add_queueout(self, data):
    if len(data):
      self.q_outLock.acquire()
      self.q_out.put(data)
      self.q_outLock.release()
      self.wakeup()

  def check_enter(self):
    for input in enumerate(self.inputs):
      found = False
      for func in self.funcs:
        m = re.match(func[0], input[1])
        if m:
          log.debug("Match found with [%s], calling %s" % (input[1], func[1]))
          if func[1]:
            func[1](self, m)
            found = True
      if found:
        self.inputs.pop(input[0])

  def recv(self, data):
    if self.running:
      self.add_queuein(data)

  def stop(self):
    self.running = False
    self.wakeup()

  def wakeup(self):
    log.debug("Waking up")
    self.waitLock.acquire()
    self.waitLock.notify()
    self.waitLock.release()
    self.wait = False

  def nothing_to_do(self):
    self.q_outLock.acquire()
    qout = not self.q_out.empty()
    self.q_outLock.release()
    self.q_inLock.acquire()
    qin = not self.q_in.empty()
    self.q_inLock.release()
    wait = qout or qin or self.wait
    return (not wait)

  def run(self):
    log.debug("Starting")
    self.running = True

    while self.running:
      self.q_inLock.acquire()
      if not self.q_in.empty():
        data = self.q_in.get()
        self.q_inLock.release()
        self.process_in(data)
      else:
        self.q_inLock.release()

      self.q_outLock.acquire()
      if not self.q_out.empty():
        data = self.q_out.get()
        self.q_outLock.release()
        self.process_out(data)
      else:
        self.q_outLock.release()

      self.check_enter()
      if self.nothing_to_do():
        self.waitLock.acquire()
        self.waitLock.wait()
        self.waitLock.release()

    self.socketThread.running = False
    log.debug("Exiting")
    pass
